( cd usr/lib ; rm -rf libmetacity-private.so.0 )
( cd usr/lib ; ln -sf libmetacity-private.so.0.0.0 libmetacity-private.so.0 )
( cd usr/lib ; rm -rf libmetacity-private.so )
( cd usr/lib ; ln -sf libmetacity-private.so.0.0.0 libmetacity-private.so )
